@@ -461,7 +461,8 @@ feed_get_node_type (void)
 		NODE_CAPABILITY_SHOW_UNREAD_COUNT |
 		NODE_CAPABILITY_UPDATE |
 		NODE_CAPABILITY_UPDATE_FAVICON |
-		NODE_CAPABILITY_EXPORT,
+		NODE_CAPABILITY_EXPORT |
+		NODE_CAPABILITY_EXPORT_ITEMS,
 		"feed",		/* not used, feed format ids are used instead */
 		NULL,
 		feed_import,
