@@ -342,7 +342,6 @@ feedlist_mark_all_read (nodePtr node)
 		node_foreach_child (ROOTNODE, node_mark_all_read);
 
 	feedlist_foreach (feedlist_update_node_counters);
-	itemview_select_item (NULL);
 	itemview_update_all_items ();
 	itemview_update ();
 }
@@ -497,7 +496,7 @@ feedlist_collect_unread (nodePtr node, gpointer user_data)
 	}
 	if (!node->subscription)
 		return;
-	if (!node->unreadCount && !g_str_equal (node->id, SELECTED->id))
+	if (!node->unreadCount && SELECTED && !g_str_equal (node->id, SELECTED->id))
 		return;
 
 	*list = g_slist_append (*list, g_strdup (node->id));
