@@ -92,8 +92,9 @@ node_source_type_register (nodeSourceTypePtr type)
 		g_assert (type->api.add_subscription_post);
 		g_assert (type->api.remove_subscription);
 		g_assert (type->api.remove_subscription_post);
-		g_assert (type->api.edit_add_label);
+		g_assert (type->api.edit_label);
 		g_assert (type->api.edit_add_label_post);
+		g_assert (type->api.edit_remove_label_post);
 		g_assert (type->api.edit_tag);
 		g_assert (type->api.edit_tag_add_post);
 		g_assert (type->api.edit_tag_remove_post);
