@@ -1,5 +1,5 @@
 /**
- * @file webkit.c  WebKit2 browser module for Liferea
+ * @file webkit.c  WebKit2 support for Liferea
  *
  * Copyright (C) 2016-2019 Leiaz <leiaz@mailbox.org>
  * Copyright (C) 2007-2021 Lars Windolf <lars.windolf@gmx.de>
@@ -21,8 +21,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+ 
+#include "webkit/webkit.h"
 
-#include <webkit2/webkit2.h>
 #include <string.h>
 #include <math.h>
 
@@ -32,65 +33,45 @@
 #include "enclosure.h" /* Only for enclosure_download */
 #include "render.h"
 #include "ui/browser_tabs.h"
-#include "ui/liferea_htmlview.h"
+#include "ui/liferea_browser.h"
 
 #include "web_extension/liferea_web_extension_names.h"
 #include "liferea_web_view.h"
 
-#define LIFEREA_TYPE_WEBKIT_IMPL liferea_webkit_impl_get_type ()
-
-#define LIFEREA_WEBKIT_IMPL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), LIFEREA_TYPE_WEBKIT_IMPL, LifereaWebKitImpl))
-#define IS_LIFEREA_WEBKIT_IMPL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), LIFEREA_TYPE_WEBKIT_IMPL))
-#define LIFEREA_WEBKIT_IMPL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), LIFEREA_TYPE_WEBKIT_IMPL, LifereaWebKitImplClass))
-#define IS_LIFEREA_WEBKIT_IMPL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), LIFEREA_TYPE_WEBKIT_IMPL))
-#define LIFEREA_WEBKIT_IMPL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), LIFEREA_TYPE_WEBKIT_IMPL, LifereaWebKitImplClass))
-
-
-typedef struct _LifereaWebKitImpl {
-	GObject parent;
-
-	GDBusServer 	*dbus_server;
-	GList 		*dbus_connections;
-} LifereaWebKitImpl;
-
-typedef struct _LifereaWebKitImplClass {
-	GObjectClass parent_class;
-} LifereaWebKitImplClass;
-
-G_DEFINE_TYPE (LifereaWebKitImpl, liferea_webkit_impl, G_TYPE_OBJECT)
+G_DEFINE_TYPE (LifereaWebKit, liferea_webkit, G_TYPE_OBJECT)
 
 // singleton
-LifereaWebKitImpl *impl = NULL;
+LifereaWebKit *webkit = NULL;
 
 enum {
 	PAGE_CREATED_SIGNAL,
 	N_SIGNALS
 };
 
-static guint liferea_webkit_impl_signals [N_SIGNALS];
+static guint liferea_webkit_signals [N_SIGNALS];
 
 static void
-liferea_webkit_impl_dispose (GObject *gobject)
+liferea_webkit_dispose (GObject *gobject)
 {
-	LifereaWebKitImpl *self = LIFEREA_WEBKIT_IMPL(gobject);
+	LifereaWebKit *self = LIFEREA_WEBKIT(gobject);
 
 	g_clear_object (&self->dbus_server);
 	g_list_free_full (self->dbus_connections, g_object_unref);
 
 	/* Chaining dispose from parent class. */
-	G_OBJECT_CLASS(liferea_webkit_impl_parent_class)->dispose(gobject);
+	G_OBJECT_CLASS(liferea_webkit_parent_class)->dispose(gobject);
 }
 
 static void
-liferea_webkit_impl_class_init (LifereaWebKitImplClass *klass)
+liferea_webkit_class_init (LifereaWebKitClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
 	GType signal_params[2] = {G_TYPE_POINTER, G_TYPE_UINT64};
 
-	liferea_webkit_impl_signals[PAGE_CREATED_SIGNAL] = g_signal_newv (
+	liferea_webkit_signals[PAGE_CREATED_SIGNAL] = g_signal_newv (
 		"page-created",
-		LIFEREA_TYPE_WEBKIT_IMPL,
+		LIFEREA_TYPE_WEBKIT,
 		G_SIGNAL_RUN_FIRST,
 		NULL,
 		NULL,
@@ -100,14 +81,7 @@ liferea_webkit_impl_class_init (LifereaWebKitImplClass *klass)
 		2,
 		signal_params);
 
-	gobject_class->dispose = liferea_webkit_impl_dispose;
-}
-
-
-static LifereaWebKitImpl *
-liferea_webkit_impl_new (void)
-{
-	return g_object_new (LIFEREA_TYPE_WEBKIT_IMPL, NULL);
+	gobject_class->dispose = liferea_webkit_dispose;
 }
 
 /**
@@ -267,7 +241,7 @@ liferea_webkit_on_dbus_connection_close (GDBusConnection *connection,
 					 GError          *error,
 					 gpointer        user_data)
 {
-	LifereaWebKitImpl *webkit_impl = LIFEREA_WEBKIT_IMPL (user_data);
+	LifereaWebKit *webkit_impl = LIFEREA_WEBKIT (user_data);
 
 	if (!remote_peer_vanished && error)
 	{
@@ -287,18 +261,18 @@ liferea_webkit_emit_page_created (GDBusConnection *connection,
 			     gpointer user_data)
 {
 	guint64 page_id;
-	LifereaWebKitImpl *webkit_impl = LIFEREA_WEBKIT_IMPL (user_data);
+	LifereaWebKit *webkit_impl = LIFEREA_WEBKIT (user_data);
 
 	g_variant_get (parameters, "(t)", &page_id);
 	g_signal_emit (webkit_impl,
-		liferea_webkit_impl_signals[PAGE_CREATED_SIGNAL],
+		liferea_webkit_signals[PAGE_CREATED_SIGNAL],
 		0,
 		(gpointer) connection,
 		page_id);
 }
 
 static void
-on_page_created (LifereaWebKitImpl *instance,
+on_page_created (LifereaWebKit *instance,
 		 GDBusConnection *connection,
 		 guint64 page_id,
 		 gpointer web_view)
@@ -312,7 +286,7 @@ on_page_created (LifereaWebKitImpl *instance,
 static gboolean
 liferea_webkit_on_new_dbus_connection (GDBusServer *server, GDBusConnection *connection, gpointer user_data)
 {
-	LifereaWebKitImpl *webkit_impl = LIFEREA_WEBKIT_IMPL (user_data);
+	LifereaWebKit *webkit_impl = LIFEREA_WEBKIT (user_data);
 
 	webkit_impl->dbus_connections = g_list_append (webkit_impl->dbus_connections, g_object_ref (connection));
 	g_signal_connect (connection,
@@ -344,7 +318,7 @@ liferea_webkit_initialize_web_extensions (WebKitWebContext 	*context,
 	gchar 	*server_address = NULL;
 	GError	*error = NULL;
 	GDBusAuthObserver *observer = NULL;
-	LifereaWebKitImpl *webkit_impl = LIFEREA_WEBKIT_IMPL (user_data);
+	LifereaWebKit *webkit_impl = LIFEREA_WEBKIT (user_data);
 
 	guid = g_dbus_generate_guid ();
 	address = g_strdup_printf ("unix:tmpdir=%s", g_get_tmp_dir ());
@@ -382,7 +356,7 @@ liferea_webkit_initialize_web_extensions (WebKitWebContext 	*context,
 }
 
 static void
-liferea_webkit_impl_download_started (WebKitWebContext	*context,
+liferea_webkit_download_started (WebKitWebContext	*context,
 				      WebKitDownload 	*download,
 				      gpointer 		user_data)
 {
@@ -407,11 +381,12 @@ liferea_webkit_handle_liferea_scheme (WebKitURISchemeRequest *request, gpointer
 }
 
 static void
-liferea_webkit_impl_init (LifereaWebKitImpl *self)
+liferea_webkit_init (LifereaWebKit *self)
 {
-	gboolean	enable_itp;
-	WebKitSecurityManager *security_manager;
-	WebKitWebsiteDataManager *website_data_manager;
+	gboolean			enable_itp;
+	WebKitSecurityManager		*security_manager;
+	WebKitWebsiteDataManager	*website_data_manager;
+		
 	self->dbus_connections = NULL;
 	webkit_web_context_register_uri_scheme (webkit_web_context_get_default(), "liferea",
 		(WebKitURISchemeRequestCallback) liferea_webkit_handle_liferea_scheme,NULL,NULL);
@@ -441,32 +416,17 @@ liferea_webkit_impl_init (LifereaWebKitImpl *self)
 	g_signal_connect (
 		webkit_web_context_get_default (),
 		"download-started",
-		G_CALLBACK (liferea_webkit_impl_download_started),
+		G_CALLBACK (liferea_webkit_download_started),
 		self);
 }
 
-
-static LifereaWebKitImpl *liferea_webkit_impl = NULL;
-
-/**
- * HTML renderer init method
- */
-static void
-liferea_webkit_init (void)
-{
-	g_assert (!liferea_webkit_impl);
-
-	liferea_webkit_impl = liferea_webkit_impl_new ();
-}
-
-
 /**
  * Load HTML string into the rendering scrollpane
  *
  * Load an HTML string into the web view. This is used to render
  * HTML documents created internally.
  */
-static void
+void
 liferea_webkit_write_html (
 	GtkWidget *webview,
 	const gchar *string,
@@ -491,7 +451,7 @@ liferea_webkit_write_html (
 	g_bytes_unref (string_bytes);
 }
 
-static void
+void
 liferea_webkit_run_js (GtkWidget *widget, gchar *js)
 {
 	// No matter what was before we need JS now
@@ -563,17 +523,20 @@ liferea_webkit_default_settings (WebKitSettings *settings)
 	);
 }
 
+static LifereaWebKit *liferea_webkit = NULL;
+
 /**
- * Initializes WebKit
- *
- * Initializes the WebKit HTML rendering engine. Creates a WebKitWebView.
+ * Create new WebkitWebView object and connect signals to a LifereaHtmlview
  */
-static GtkWidget *
-liferea_webkit_new (LifereaHtmlView *htmlview)
+GtkWidget *
+liferea_webkit_new (LifereaBrowser *htmlview)
 {
 	WebKitWebView 	*view;
 	WebKitSettings	*settings;
 
+	if (!liferea_webkit)
+		liferea_webkit = g_object_new (LIFEREA_TYPE_WEBKIT, NULL);
+
 	view = WEBKIT_WEB_VIEW (liferea_web_view_new ());
 
 	settings = webkit_settings_new ();
@@ -581,30 +544,27 @@ liferea_webkit_new (LifereaHtmlView *htmlview)
 	webkit_web_view_set_settings (view, settings);
 
 	g_signal_connect_object (
-		liferea_webkit_impl,
+		liferea_webkit,
 		"page-created",
 		G_CALLBACK (on_page_created),
 		view,
 		G_CONNECT_AFTER);
 
-	/** Pass LifereaHtmlView into the WebKitWebView object */
+	/** Pass LifereaBrowser into the WebKitWebView object */
 	g_object_set_data (
 		G_OBJECT (view),
 		"htmlview",
 		htmlview
 	);
 
-	g_signal_connect (G_OBJECT (view), "screen_changed", G_CALLBACK (liferea_webkit_screen_changed), NULL);
+	g_signal_connect (G_OBJECT (view), "screen_changed", G_CALLBACK (liferea_webkit_screen_changed), settings);
 	g_signal_connect (G_OBJECT (view), "realize", G_CALLBACK (liferea_webkit_set_font_size), settings);
 
 	gtk_widget_show (GTK_WIDGET (view));
 	return GTK_WIDGET (view);
 }
 
-/**
- * Launch URL
- */
-static void
+void
 liferea_webkit_launch_url (GtkWidget *webview, const gchar *url)
 {
 	// FIXME: hack to make URIs like "gnome.org" work
@@ -628,43 +588,31 @@ liferea_webkit_launch_url (GtkWidget *webview, const gchar *url)
 	g_free (http_url);
 }
 
-/**
- * Change zoom level of the HTML scrollpane
- */
-static void
+void
 liferea_webkit_change_zoom_level (GtkWidget *webview, gfloat zoom_level)
 {
 	webkit_web_view_set_zoom_level (WEBKIT_WEB_VIEW (webview), zoom_level);
 }
 
-/**
- * Copy selected text to the clipboard
- */
-static void
-liferea_webkit_copy_selection (GtkWidget *webview)
-{
-	webkit_web_view_execute_editing_command (WEBKIT_WEB_VIEW (webview), WEBKIT_EDITING_COMMAND_COPY);
-}
-
-/**
- * Return current zoom level as a float
- */
-static gfloat
+gfloat
 liferea_webkit_get_zoom_level (GtkWidget *webview)
 {
 	return webkit_web_view_get_zoom_level (WEBKIT_WEB_VIEW (webview));
 }
 
-/**
- * Scroll page down (via shortcut key)
- */
-static void
+void
+liferea_webkit_copy_selection (GtkWidget *webview)
+{
+	webkit_web_view_execute_editing_command (WEBKIT_WEB_VIEW (webview), WEBKIT_EDITING_COMMAND_COPY);
+}
+
+void
 liferea_webkit_scroll_pagedown (GtkWidget *webview)
 {
 	liferea_web_view_scroll_pagedown (LIFEREA_WEB_VIEW (webview));
 }
 
-static void
+void
 liferea_webkit_set_proxy (ProxyDetectMode mode, const gchar *host, guint port, const gchar *user, const gchar *pwd)
 {
 #if WEBKIT_CHECK_VERSION (2, 15, 3)
@@ -717,20 +665,27 @@ liferea_webkit_set_proxy (ProxyDetectMode mode, const gchar *host, guint port, c
 #endif
 }
 
+static gchar *lastCss = NULL;
+
 /**
  * Load liferea.css via user style sheet
  */
-static void
-liferea_webkit_set_style (GtkWidget *webview)
+void
+liferea_webkit_reload_style (GtkWidget *webview)
 {
-	if (render_get_css () == NULL)
+	gchar *css = render_get_css ();
+
+	if (css == NULL || g_strcmp0 (css, lastCss) == 0)
 		return;
 
+	g_free (lastCss);
+	lastCss = css;
+
 	WebKitUserContentManager *manager = webkit_web_view_get_user_content_manager (WEBKIT_WEB_VIEW (webview));
 
 	webkit_user_content_manager_remove_all_style_sheets (manager);
 
-	WebKitUserStyleSheet *stylesheet = webkit_user_style_sheet_new (render_get_css(),
+	WebKitUserStyleSheet *stylesheet = webkit_user_style_sheet_new (css,
 		WEBKIT_USER_CONTENT_INJECT_ALL_FRAMES,
 		WEBKIT_USER_STYLE_LEVEL_USER,
 		NULL,
@@ -742,30 +697,10 @@ liferea_webkit_set_style (GtkWidget *webview)
 /**
  * Reload the current contents of webview
  */
-static void
+void
 liferea_webkit_reload (GtkWidget *webview)
 {
 	liferea_webkit_default_settings (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (webview)));
 
-	webkit_web_view_reload (webview);
+	webkit_web_view_reload (WEBKIT_WEB_VIEW (webview));
 }
-
-static struct
-htmlviewImpl webkitImpl = {
-	.init		= liferea_webkit_init,
-	.create		= liferea_webkit_new,
-	.write		= liferea_webkit_write_html,
-	.run_js		= liferea_webkit_run_js,
-	.launch		= liferea_webkit_launch_url,
-	.zoomLevelGet	= liferea_webkit_get_zoom_level,
-	.zoomLevelSet	= liferea_webkit_change_zoom_level,
-	.hasSelection	= NULL,  /* Was only useful for the context menu, can be removed */
-	.copySelection	= liferea_webkit_copy_selection, /* Same. */
-	.scrollPagedown	= liferea_webkit_scroll_pagedown,
-	.setProxy	= liferea_webkit_set_proxy,
-	.setOffLine	= NULL, // FIXME: blocked on https://bugs.webkit.org/show_bug.cgi?id=18893
-	.setStylesheet	= liferea_webkit_set_style,
-	.reload		= liferea_webkit_reload
-};
-
-DECLARE_HTMLVIEW_IMPL (webkitImpl);
